! until better Jam with target and projetile break up make next
! to neglect  target sepctator nucleonss. 
#define NEGTARGETSPEC

      subroutine cjamEvent(pj, ia, iz, sig, a, ntp)
      implicit none
#include "Zcode.h"      
#include "Zptcl.h"      
      record /ptcl/ pj  !  input. projectile

      integer,intent(in)::ia, iz ! target A and Z
      real(8),intent(in):: sig  ! cross seccion in mb on this target
      record /ptcl/ a(*)  !output. generated ptcls
      integer,intent(out):: ntp  ! # of generated ptcls

      logical ok
      integer inela, loop
      record /ptcl/ tg  !  

      if(ia == 1 ) then
         call cmkptc(knuc, -1, iz, tg)
      elseif(ia > 1 ) then
         call cmkptc(kgnuc, ia, iz, tg)
      else
         write(0,*) 'in cjamEvent: strange target A,Z',ia,iz
         stop
      endif
      tg.fm.p(4) = tg.mass
      tg.fm.p(1:3)= 0.

      ok = .false.
      call cjamini(pj, tg, sig)
      do while (.not. ok)
         call cjam(pj, a, ntp, inela)
c          jam cannot disable elastic collision.
c        for non heavy, we can use total x-section
c        which includes elastic one, but for heavy
c        we cannot get elastic cross-section. So we have to 
c          wait until inelastic collsion for heavy.
c        
         ok = pj.code /= kgnuc .or.  inela /= 0
      enddo
      call cjamElaInfo(0, inela)  ! inform inela
      call crot3mom( pj, a, ntp )   ! rotate to the current  cooord
      end
      subroutine cjamElaInfo(inout, inela)
      implicit none
      integer,intent(in)::   inout ! 0--> inela is input
                                   ! 1--> //    output
      integer,intent(inout):: inela  ! 0--> jam interaction is elastic
                                  !  1--> //                  inelastic
      integer,save::saveinela  ! keep input inela
      
      if(inout == 0) then
         saveinela = inela
      else
         inela = saveinela
      endif
      end

      subroutine cjamini(pj, tg, sig)
c      This must be called before generating one exclusive 
c      event (call jamev) 
c      implicit none  ! this cannot be used since jam1.inc uses
c      implicit double (...)
#include "jam1.inc"
#include "jam2.inc"

#include "Zptcl.h"
#include "Zkfcode.h"
#include "Zmanagerp.h"
#include "Zevhnp.h"
#include "Zevhnv.h"
#include "Zcode.h"
#include "Zmass.h"
      record /ptcl /pj, tg  ! input 
      real(8),intent(in)::sig  ! input  cross-sction on target tg in mb
                         ! used to calculate the impact parameter

      integer ia, iz    ! input target nucleus

      character frame*8, proj*8, targ*8, cwin*15
      real*8 bmin, bmax, dt
      integer mevent, nstep
      integer first /1/
      save
      
      if(first .eq. 1) then
c//////////:  this call may unresolved ref. for utilities in Epics
c         call mydummy
c////////
         first = 0
         mevent = 1
         bmin = 0.
         bmax = -7.50D0           ! maximum impact parameter (1530mb)Au  
c            for pp collison. bmax is not used.?
c            for AA bmax=0 make loop
         dt = 100.d0
         nstep =1
         mstc(8) = 0       ! job mode
         mstc(156) = 0     ! no call to jamanacl
         mstc(161) = 0     ! no output on JAMINFO.DAT, JAMRUN.DAT        
         mstc(39) = 0     ! no ouptut on JAMMULTI.DAT  
         mstc(81)=0  ! neglect hard scattering (for meson)
         frame = 'lab'
         fname(1) = '0'
         fname(2) = '/dev/null'
         fname(3) = '/dev/null'
      endif
      bmax = - sqrt( sig/3.141592/10. )  ! in fm  must be < 0
      call ccos2jamsymbl(pj, proj)  ! make projectile symbol
c               such as proj = 'k+'
      call ccos2jamsymbl(tg, targ)

c	targ = '14:7'
c	targ = 'p'
c	targ = 'n'
c	targ='197Au   '
c	targ = '4:2'
c	targ = '2:1'
      if(pj.code == kgnuc ) then
         write(cwin, '(f12.3,"gev")')  (pj.fm.p(4)-pj.mass)/pj.subcode
      else
         write(cwin, '(f12.3,"gev")')  (pj.fm.p(4)-pj.mass)
      endif
c      call kseblk(cwin, "*", nc)
c      cwin = '14.6gevc'
c///////////
c      write(0,*) ' jaminit bmax=',bmax, ' sig=',sig, ' targ=',targ,
c     *   'proj=',proj
c//////////////
      call jaminit(mevent, bmin, bmax, dt, nstep, frame, proj, targ,
     *  cwin)
      end
      subroutine  ccos2jamsymbl(pj, symb)
      implicit none
#include "Zptcl.h"
#include "Zcode.h"
      record /ptcl/ pj  ! input. code, subcode, charge 
                        ! iouput. incase of Lambda, pj is
                        ! modified to be a neutron
      character*8 symb  ! output. symbol used by jam

      integer nc

      if(pj.code .eq. kpion) then
         if(pj.charge .eq. 1) then
            symb = 'pi+'
         elseif(pj.charge .eq. 0) then
            write(0,*)
     *       'pi0 cannot be col. projectile in jam'
            stop
         else
            symb = 'pi-'
         endif
      elseif(pj.code .eq. kkaon) then
         if(pj.charge .eq. 1) then
            symb = 'k+'
         elseif(pj.charge .eq. -1) then
            symb = 'k-'
         else
           symb = 'k0'
c            write(0,*)  ' k0 cannot be proj. in jam'
c            stop
         endif
      elseif(pj.code .eq. knuc ) then
         if(pj.charge .eq. 1) then
            symb = 'p'
         elseif(pj.charge .eq.  -1 ) then
            symb = 'pbar'
         elseif(pj.subcode .eq. regptcl) then
            symb = 'n'
         else
            symb = 'nbar'
         endif
      elseif(pj.code .eq.kgnuc) then
         write(symb, '(i3,":", i3)' ) pj.subcode, 
     *       pj.subcode-pj.charge
c            remove blanks
         call kseblk(symb, "*", nc)  !  A:N say 207:125
      elseif( pj.code >= klambda .and. pj.code <= klambdac ) then
c          regard it as neutron                                                
         call cmkptc(knuc, -1, 0, pj)
         call ce2p(pj)
         symb = 'n'
      elseif( pj.code == kbomega ) then
c         very rare: regard it as proton
         call cmkptc(knuc, -1, 1, pj)
         call ce2p(pj)
         symb = 'p'
      else
         write(0,*) 'code=',pj.code, ' charge=',pj.charge
         write(0,*) ' not supported in jam'
         stop
      endif
      end
      subroutine cjam(pj, a, ntp, inela)
c      implicit none  ! this cannot be used since jam1.inc uses
c      implicit double (...)
#include "jam1.inc"
#include "jam2.inc"
#include "Zptcl.h"

      integer ntp
      record /ptcl/pj
      record /ptcl/a(*)
      integer,intent(out):: inela ! =0  elastic event
                                  ! =1  inelastic event
      integer i
      integer code, subcode, charge
      integer iev/1/
      save
c///////////
cc            these are to see the collsion of incident
cc            particles only
c      record /ptcl/ inci
c      integer,save::first=0
c      real*8  tgTE, pjTE
c      if( first == 0 ) then
c         inci = pj
c         first = 1
c      endif
c///////////

      call jamevt(iev)
      ntp = 0
      inela = 0
c////////////
c      tgTE =0.
c      pjTE = 0.
c///////////
      do i = 1, nv
         if( k(1,i) <= 11 .and. k(1,i) >= 0  ) then
c//////////
c            if( pj.fm.p(4) > inci.fm.p(4)*0.98) then
c               call ckf2cos(k(2,i), code, subcode, charge)
c               write(*,'("***",4i4)') i, code,subcode, charge
c               write(*,'("k(1:7)=",7i8)') k(1:7,i)
c               write(*,'("px,y,z,E=",1p,4g12.3)') p(1:4,i)
c               if( k(7,i) < 0 )  then
c                  pjTE = pjTE + p(4,i)
c               else
c                  tgTE = tgTE + p(4,i) 
c               endif
c            endif
c ///////////////////////// should be modifed in future
#ifdef NEGTARGETSPEC
c               Nara san's spec is -1 for target but seems to be
c               incorrect and 1 must be used
            if( k(7,i) == 1 ) cycle ! target spectator is neglected
#endif
            if( abs(k(7,i)) > 1 ) then
               inela = 1
            endif

            call ckf2cos(k(2,i), code, subcode, charge)
            ntp = ntp + 1
            call cmkptc(code, subcode, charge, a(ntp))
            a(ntp).fm.p(1) = p(1,i)
            a(ntp).fm.p(2) = p(2,i)
            a(ntp).fm.p(3) = p(3,i)
            a(ntp).fm.p(4) = p(4,i)  ! jam is T.E  (K.E is false).

c            write(*,*) i, k(2,i), p(1,i), p(2,i), p(3,i),
c     *             p(4,i), p(5,i)
         endif
      enddo
c///////////////
c      if( pj.fm.p(4) > inci.fm.p(4)*0.98 ) then
c         write(*,*) 'pjTE=',pjTE, ' pj. =',pj.fm.p(4)
c         write(*,*) 'tgKE=',tgTE, ' Mt=', pjTE+tgTE-pj.fm.p(4)
c         write(*,*)
c         write(*,*) "====================="
c         write(*,*)
c      endif
c/////////////////
      end
